@@ -1,9 +1,9 @@
 //
-// "$Id: Fl_x.cxx,v 1.24.2.8 1999/11/10 09:18:31 bill Exp $"
+// "$Id: Fl_x.cxx,v 1.24.2.18 2000/06/18 00:38:41 bill Exp $"
 //
 // X specific code for the Fast Light Tool Kit (FLTK).
 //
-// Copyright 1998-1999 by Bill Spitzak and others.
+// Copyright 1998-2000 by Bill Spitzak and others.
 //
 // This library is free software; you can redistribute it and/or
 // modify it under the terms of the GNU Library General Public
@@ -20,7 +20,7 @@
 // Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
 // USA.
 //
-// Please report all bugs and problems to "fltk-bugs@easysw.com".
+// Please report all bugs and problems to "fltk-bugs@fltk.org".
 //
 
 #ifdef WIN32
@@ -45,7 +45,7 @@
 ////////////////////////////////////////////////////////////////
 // interface to poll/select call:
 
-#if HAVE_POLL
+#if USE_POLL
 
 #include <poll.h>
 static pollfd *pollfds = 0;
@@ -65,13 +65,15 @@ static int maxfd;
 #define POLLOUT 4
 #define POLLERR 8
 
-#endif /* HAVE_POLL */
+#endif /* USE_POLL */
 
 static int nfds = 0;
 static int fd_array_size = 0;
 static struct FD {
+#if !USE_POLL
   int fd;
   short events;
+#endif
   void (*cb)(int, void*);
   void* arg;
 } *fd = 0;
@@ -82,18 +84,18 @@ void Fl::add_fd(int n, int events, void (*cb)(int, void*), void *v) {
   if (i >= fd_array_size) {
     fd_array_size = 2*fd_array_size+1;
     fd = (FD*)realloc(fd, fd_array_size*sizeof(FD));
-#if HAVE_POLL
+#if USE_POLL
     pollfds = (pollfd*)realloc(pollfds, fd_array_size*sizeof(pollfd));
 #endif
   }
-  fd[i].fd = n;
-  fd[i].events = events;
   fd[i].cb = cb;
   fd[i].arg = v;
-#if HAVE_POLL
-  fds[i].fd = n;
-  fds[i].events = events;
+#if USE_POLL
+  pollfds[i].fd = n;
+  pollfds[i].events = events;
 #else
+  fd[i].fd = n;
+  fd[i].events = events;
   if (events & POLLIN) FD_SET(n, &fdsets[0]);
   if (events & POLLOUT) FD_SET(n, &fdsets[1]);
   if (events & POLLERR) FD_SET(n, &fdsets[2]);
@@ -108,25 +110,30 @@ void Fl::add_fd(int fd, void (*cb)(int, void*), void* v) {
 void Fl::remove_fd(int n, int events) {
   int i,j;
   for (i=j=0; i<nfds; i++) {
+#if USE_POLL
+    if (pollfds[i].fd == n) {
+      int e = pollfds[i].events & ~events;
+      if (!e) continue; // if no events left, delete this fd
+      pollfds[j].events = e;
+    }
+#else
     if (fd[i].fd == n) {
       int e = fd[i].events & ~events;
       if (!e) continue; // if no events left, delete this fd
       fd[i].events = e;
-#if HAVE_POLL
-      fds[j].events = e;
-#endif
     }
+#endif
     // move it down in the array if necessary:
     if (j<i) {
-      fd[j]=fd[i];
-#if HAVE_POLL
-      fds[j]=fds[i];
+      fd[j] = fd[i];
+#if USE_POLL
+      pollfds[j] = pollfds[i];
 #endif
     }
     j++;
   }
   nfds = j;
-#if !HAVE_POLL
+#if !USE_POLL
   if (events & POLLIN) FD_CLR(n, &fdsets[0]);
   if (events & POLLOUT) FD_CLR(n, &fdsets[1]);
   if (events & POLLERR) FD_CLR(n, &fdsets[2]);
@@ -138,22 +145,6 @@ void Fl::remove_fd(int n) {
   remove_fd(n, -1);
 }
 
-int fl_ready() {
-  if (XQLength(fl_display)) return 1;
-#if HAVE_POLL
-  return ::poll(fds, nfds, 0);
-#else
-  timeval t;
-  t.tv_sec = 0;
-  t.tv_usec = 0;
-  fd_set fdt[3];
-  fdt[0] = fdsets[0];
-  fdt[1] = fdsets[1];
-  fdt[2] = fdsets[2];
-  return ::select(maxfd+1,&fdt[0],&fdt[1],&fdt[2],&t);
-#endif
-}
-
 #if CONSOLIDATE_MOTION
 static Fl_Window* send_motion;
 extern Fl_Window* fl_xmousewin;
@@ -172,14 +163,17 @@ static void do_queued_events() {
 #endif
 }
 
-double fl_wait(int timeout_flag, double time) {
+// This is never called with time_to_wait < 0.0:
+// It should return negative on error, 0 if nothing happens before
+// timeout, and >0 if any callbacks were done.
+int fl_wait(double time_to_wait) {
 
   // OpenGL and other broken libraries call XEventsQueued
   // unnecessarily and thus cause the file descriptor to not be ready,
   // so we must check for already-read events:
-  if (XQLength(fl_display)) {do_queued_events(); return time;}
+  if (fl_display && XQLength(fl_display)) {do_queued_events(); return 1;}
 
-#if !HAVE_POLL
+#if !USE_POLL
   fd_set fdt[3];
   fdt[0] = fdsets[0];
   fdt[1] = fdsets[1];
@@ -187,31 +181,26 @@ double fl_wait(int timeout_flag, double time) {
 #endif
   int n;
 
-  if (!timeout_flag) {
-#if HAVE_POLL
-    n = ::poll(fds, nfds, -1);
-#else
-    n = ::select(maxfd+1,&fdt[0],&fdt[1],&fdt[2],0);
-#endif
-  } else {
-#if HAVE_POLL
-    int n = ::poll(fds, nfds, time > 0.0 ? int(time*1000) : 0);
+  if (time_to_wait < 2147483.648) {
+#if USE_POLL
+    n = ::poll(pollfds, nfds, int(time_to_wait*1000 + .5));
 #else
     timeval t;
-    if (time <= 0.0) {
-      t.tv_sec = 0;
-      t.tv_usec = 0;
-    } else {
-      t.tv_sec = int(time);
-      t.tv_usec = int(1000000 * (time-t.tv_sec));
-    }
+    t.tv_sec = int(time_to_wait);
+    t.tv_usec = int(1000000 * (time_to_wait-t.tv_sec));
     n = ::select(maxfd+1,&fdt[0],&fdt[1],&fdt[2],&t);
+#endif
+  } else {
+#if USE_POLL
+    n = ::poll(pollfds, nfds, -1);
+#else
+    n = ::select(maxfd+1,&fdt[0],&fdt[1],&fdt[2],0);
 #endif
   }
   if (n > 0) {
     for (int i=0; i<nfds; i++) {
-#if HAVE_POLL
-      if (fds[i].revents) fd[i].cb(fd[i].fd, fd[i].arg);
+#if USE_POLL
+      if (pollfds[i].revents) fd[i].cb(pollfds[i].fd, fd[i].arg);
 #else
       int f = fd[i].fd;
       short revents = 0;
@@ -222,7 +211,24 @@ double fl_wait(int timeout_flag, double time) {
 #endif
     }
   }
-  return time;
+  return n;
+}
+
+// fl_ready() is just like fl_wait(0.0) except no callbacks are done:
+int fl_ready() {
+  if (XQLength(fl_display)) return 1;
+#if USE_POLL
+  return ::poll(pollfds, nfds, 0);
+#else
+  timeval t;
+  t.tv_sec = 0;
+  t.tv_usec = 0;
+  fd_set fdt[3];
+  fdt[0] = fdsets[0];
+  fdt[1] = fdsets[1];
+  fdt[2] = fdsets[2];
+  return ::select(maxfd+1,&fdt[0],&fdt[1],&fdt[2],&t);
+#endif
 }
 
 ////////////////////////////////////////////////////////////////
@@ -258,6 +264,10 @@ void fl_open_display() {
   Display *d = XOpenDisplay(0);
   if (!d) Fl::fatal("Can't open display: %s",XDisplayName(0));
 
+  fl_open_display(d);
+}
+
+void fl_open_display(Display* d) {
   fl_display = d;
 
   wm_delete_window = XInternAtom(d,"WM_DELETE_WINDOW",0);
@@ -271,6 +281,10 @@ void fl_open_display() {
   templt.visualid = XVisualIDFromVisual(DefaultVisual(fl_display,fl_screen));
   fl_visual = XGetVisualInfo(fl_display, VisualIDMask, &templt, &num);
   fl_colormap = DefaultColormap(fl_display,fl_screen);
+
+#if !USE_COLORMAP
+  Fl::visual(FL_RGB);
+#endif
 }
 
 void fl_close_display() {
@@ -348,6 +362,7 @@ static Fl_Window* resize_bug_fix;
 
 int fl_handle(const XEvent& xevent)
 {
+  Fl::e_keysym = 0;
   fl_xevent = &xevent;
   Window xid = xevent.xany.window;
 
@@ -447,7 +462,9 @@ int fl_handle(const XEvent& xevent)
     fl_key_vector[keycode/8] |= (1 << (keycode%8));
     static char buffer[21];
     KeySym keysym;
-    int len = XLookupString((XKeyEvent*)&(xevent.xkey),buffer,20,&keysym,0);
+    //static XComposeStatus compose;
+    int len = XLookupString((XKeyEvent*)&(xevent.xkey),
+			    buffer, 20, &keysym, 0/*&compose*/);
     if (keysym && keysym < 0x400) { // a character in latin-1,2,3,4 sets
       // force it to type a character (not sure if this ever is needed):
       if (!len) {buffer[0] = char(keysym); len = 1;}
@@ -562,9 +579,10 @@ void Fl_Window::resize(int X,int Y,int W,int H) {
     x(X); y(Y);
   }
   if (resize_from_program && shown()) {
-    if (is_a_resize)
+    if (is_a_resize) {
+      if (!resizable()) size_range(w(),h(),w(),h());
       XMoveResizeWindow(fl_display, i->xid, X, Y, W>0 ? W : 1, H>0 ? H : 1);
-    else
+    } else
       XMoveWindow(fl_display, i->xid, X, Y);
   }
 }
@@ -673,9 +691,7 @@ void Fl_X::make_xid(Fl_Window* w, XVisualInfo *visual, Colormap colormap)
 			     InputOutput,
 			     visual->visual,
 			     mask, &attr));
-  w->set_visible();
-  w->handle(FL_SHOW); // get child windows to appear
-  w->redraw();
+  int showit = 1;
 
   if (!w->parent() && !attr.override_redirect) {
     // Communicate all kinds 'o junk to the X Window Manager:
@@ -708,23 +724,31 @@ void Fl_X::make_xid(Fl_Window* w, XVisualInfo *visual, Colormap colormap)
       Fl_Window* w = x->next->w;
       while (w->parent()) w = w->window();
       XSetTransientForHint(fl_display, x->xid, fl_xid(w));
+      if (!w->visible()) showit = 0; // guess that wm will not show it
     }
 
     XWMHints hints;
-    hints.flags = 0;
+    hints.input = True;
+    hints.flags = InputHint;
     if (fl_show_iconic) {
-      hints.flags = StateHint;
+      hints.flags |= StateHint;
       hints.initial_state = IconicState;
       fl_show_iconic = 0;
+      showit = 0;
     }
     if (w->icon()) {
       hints.icon_pixmap = (Pixmap)w->icon();
       hints.flags       |= IconPixmapHint;
     }
-    if (hints.flags) XSetWMHints(fl_display, x->xid, &hints);
+    XSetWMHints(fl_display, x->xid, &hints);
   }
 
   XMapWindow(fl_display, x->xid);
+  if (showit) {
+    w->set_visible();
+    w->handle(FL_SHOW); // get child windows to appear
+    w->redraw();
+  }
 }
 
 ////////////////////////////////////////////////////////////////
@@ -872,5 +896,5 @@ void Fl_Window::make_current() {
 #endif
 
 //
-// End of "$Id: Fl_x.cxx,v 1.24.2.8 1999/11/10 09:18:31 bill Exp $".
+// End of "$Id: Fl_x.cxx,v 1.24.2.18 2000/06/18 00:38:41 bill Exp $".
 //
